import isFunction from 'lodash.isfunction'
import { defaultFilterBy, isString, isObject, isNumber } from './helpers'
import { MAXIMUM_DROPDOWN_HEIGHT, UP, ACTUAL_VALUE } from './constants'

export const noop = () => { }

export const getLabelKey = (labelKey, option) => isFunction(labelKey) ? labelKey(option) : labelKey

export const getInputValue = (option) => option.__label;

export const filterResultsFromOptions = (options, {
  selected = [],
  value = '',
  filterBy,
  searchable = false,
  multiple = false,
  labelKey,
  onMenuItemRender
}) => {
  let results = [...options]
  if (!searchable && !multiple) return results
  if (searchable || multiple) {
    const filter = Array.isArray(filterBy) ? defaultFilterBy : filterBy
    results = options.filter(option => filter(option, {
      value,
      multiple,
      labelKey,
      filterBy,
      selected,
      onMenuItemRender
    }))
  }
  return results
}

export const getStateFromProps = (props) => {
  let {
    options,
    selected,
    multiple,
    labelKey,
    defaultFirstItemSelected,
    onMenuItemRender,
    searchable = false
  } = props

  let value = ''

  selected = optionsMap([...selected], { labelKey, onMenuItemRender })

  if (!multiple && selected && selected.length) {
    let _selected = [...selected].shift()
    value = getInputValue(_selected)
  }

  let _options = optionsMap([...options], { labelKey, onMenuItemRender })

  let results = filterResultsFromOptions(_options, {
    ...props,
    value
  })

  return {
    activeIndex: defaultFirstItemSelected ? 0 : -1,
    activeItem: defaultFirstItemSelected ? results[0] : null,
    options: _options,
    open: props.defaultOpen || false,
    focus: props.autoFocus || false,
    value,
    results,
    selected,
    menuPosition: {},
    autoScroll: false,
    isDirty: false,
    searchable,
    dropdownState: 'idle'
  }
}

export const constructMenuProps = props => ({
  options: [...props.results],
  labelKey: props.labelKey,
  maxDropdownHeight: MAXIMUM_DROPDOWN_HEIGHT,
  selected: props.selected,
  activeIndex: props.activeIndex,
  isDirty: props.isDirty,
  autoScroll: props.autoScroll,
  loading: props.loading,
  animate: props.animate,
  dropdownState: props.dropdownState
})

export const optionsMap = (options, props) => {
  let { labelKey, onMenuItemRender } = props
  return options.map(option => {
    let key = getLabelKey(labelKey, option)
    if (isString(option) || isNumber(option)) {
      // console.log(option)
      // console.log(isFunction(onMenuItemRender) && onMenuItemRender(option))
      return {
        [key]: option,
        __label: isFunction(onMenuItemRender) ? onMenuItemRender(option) : option,
        [ACTUAL_VALUE]: option
      }
    }
    if (isObject(option)) {
      return {
        ...option,
        [key]: option[key],
        __label: isFunction(onMenuItemRender) ? onMenuItemRender(option) : option[key],
        [ACTUAL_VALUE]: {
          ...option
        }
      }
    }
  })
}

function skipDisabledOptions(
  currentIndex,
  keyCode,
  items
){
  let newIndex = currentIndex

  while (items[newIndex] && items[newIndex].disabled) {
    newIndex += keyCode === UP ? -1 : 1
  }

  return newIndex
}

export function getUpdatedActiveIndex(
  currentIndex,
  keyCode,
  items
) {
  let newIndex = currentIndex

  // Increment or decrement index based on user keystroke.
  newIndex += keyCode === UP ? -1 : 1

  // Skip over any disabled options.
  newIndex = skipDisabledOptions(newIndex, keyCode, items)

  // If we've reached the end, go back to the beginning or vice-versa.
  if (newIndex === items.length) {
    newIndex = -1
  } else if (newIndex === -2) {
    newIndex = items.length - 1

    // Skip over any disabled options.
    newIndex = skipDisabledOptions(newIndex, keyCode, items)
  }

  return newIndex
}

export function debounce(callback, interval) {
  let debounceTimeoutId

  return function (...args) {
    clearTimeout(debounceTimeoutId)
    debounceTimeoutId = setTimeout(() => callback.apply(this, args), interval)
  }
}